#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : stream_response
# @Time         : 2023/5/26 14:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


def generate_response(content: str, chat: bool = True):
    if chat:
        return {
            "id": "chatcmpl-77PZm95TtxE0oYLRx3cxa6HtIDI7s",
            "object": "chat.completion",
            "created": 1682000966,
            "model": "gpt-3.5-turbo-0301",
            "usage": {
                "prompt_tokens": 0,
                "completion_tokens": 0,
                "total_tokens": 0,
            },
            "choices": [{
                "message": {"role": "assistant", "content": content},
                "finish_reason": "stop", "index": 0}
            ]
        }
    else:
        return {
            "id": "cmpl-uqkvlQyYK7bGYrRHQ0eXlWi7",
            "object": "text_completion",
            "created": 1589478378,
            "model": "text-davinci-003",
            "choices": [
                {
                    "text": content,
                    "index": 0,
                    "logprobs": None,
                    "finish_reason": "stop"
                }
            ],
            "usage": {
                "prompt_tokens": 0,
                "completion_tokens": 0,
                "total_tokens": 0
            }
        }


def generate_stream_response_start():
    return {
        "id": "chatcmpl-77QWpn5cxFi9sVMw56DZReDiGKmcB",
        "object": "chat.completion.chunk", "created": 1682004627,
        "model": "gpt-3.5-turbo-0301",
        "choices": [{"delta": {"role": "assistant"}, "index": 0, "finish_reason": None}]
    }


def generate_stream_response(content: str, chat: bool = True):
    if chat:
        return {
            "id": "chatcmpl-77QWpn5cxFi9sVMw56DZReDiGKmcB",
            "object": "chat.completion.chunk",
            "created": 1682004627,
            "model": "gpt-3.5-turbo-0301",
            "choices": [{"delta": {"content": content}, "index": 0, "finish_reason": None}
                        ]}
    else:
        return {
            "id": "cmpl-7GfnvmcsDmmTVbPHmTBcNqlMtaEVj",
            "object": "text_completion",
            "created": 1684208299,
            "choices": [
                {
                    "text": content,
                    "index": 0,
                    "logprobs": None,
                    "finish_reason": None,
                }
            ],
            "model": "text-davinci-003"
        }


def generate_stream_response_stop(chat: bool = True):
    if chat:
        return {"id": "chatcmpl-77QWpn5cxFi9sVMw56DZReDiGKmcB",
                "object": "chat.completion.chunk", "created": 1682004627,
                "model": "gpt-3.5-turbo-0301",
                "choices": [{"delta": {}, "index": 0, "finish_reason": "stop"}]
                }
    else:
        return {
            "id": "cmpl-7GfnvmcsDmmTVbPHmTBcNqlMtaEVj",
            "object": "text_completion",
            "created": 1684208299,
            "choices": [
                {"text": "", "index": 0, "logprobs": None, "finish_reason": "stop"}],
            "model": "text-davinci-003",
        }


if __name__ == '__main__':
    print(generate_stream_response(''))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : config
# @Time         : 2023/5/26 13:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.decorators import clear_cuda_cache

from chatllm.llms import load_llm4chat

torch_gc = clear_cuda_cache(lambda: logger.info('Clear GPU'), bins=os.getenv('TIME_INTERVAL', 15))

tokens = set(os.getenv('TOKENS', 'chatllm').split(','))
embedding_model = os.getenv('EMBEDDING_MODEL')
if embedding_model:
    from sentence_transformers import SentenceTransformer

    embedding_model = SentenceTransformer(embedding_model)
else:
    class RandomSentenceTransformer:
        def encode(self, texts):
            logger.error("请配置 EMBEDDING_MODEL")
            return np.random.random((len(texts), 64))


    embedding_model = RandomSentenceTransformer()

llm_model = os.getenv('LLM_MODEL')
device = os.getenv('DEVICE', 'cpu')
num_gpus = os.getenv('NUM_GPUS', 2)

if llm_model:  # 必配参数
    do_chat = load_llm4chat(model_name_or_path=llm_model, device=device, num_gpus=num_gpus)
    do_chat = partial(do_chat, return_history=False)
else:
    def do_chat(query, **kwargs):  # DEV
        yield from '请配置 LLM_MODEL'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatpdf
# @Time         : 2023/4/25 17:01
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import streamlit as st
from meutils.pipe import *
from meutils.serving.st_utils import display_pdf, st_chat, set_config

from chatllm.applications.chatpdf import ChatPDF

st.set_page_config(page_title='🔥ChatPDF', layout='wide', initial_sidebar_state='collapsed')


################################################################################################################
class Conf(BaseConfig):
    encode_model = 'nghuyong/ernie-3.0-nano-zh'
    llm = "THUDM/chatglm-6b"  # /Users/betterme/PycharmProjects/AI/CHAT_MODEL/chatglm-6b
    cachedir = 'pdf_cache'

    topk: int = 3
    threshold: float = 0.66


conf = Conf()
conf = set_config(conf)


################################################################################################################


@st.cache_resource()
def qa4pdf(encode_model, model_name_or_path, cachedir):
    qa = ChatPDF(encode_model=encode_model)
    # qa.encode = disk_cache(qa.encode, location=cachedir)  # 缓存
    qa.load_llm4chat(model_name_or_path=model_name_or_path)
    qa.create_index = lru_cache()(qa.create_index)

    return qa


def reply_func(query):
    for response, _ in qa(query=query, topk=conf.topk, threshold=conf.threshold):
        yield response


if st.session_state.get('init'):

    tabs = st.tabs(['ChatPDF', 'PDF文件预览'])

    with tabs[0]:
        file = st.file_uploader("上传PDF", type=['pdf'])
        bytes_array = ''
        try:
            qa = qa4pdf(conf.encode_model, conf.llm, conf.cachedir)
        except Exception as e:
            st.warning('启动前选择正确的参数进行初始化')
            st.error(e)

        if file:
            bytes_array = file.read()
            with st.spinner("构建知识库：文本向量化"):
                qa.create_index(bytes_array)

            base64_pdf = base64.b64encode(bytes_array).decode('utf-8')

            container = st.container()  # 占位符
            text = st.text_area(label="用户输入", height=100, placeholder="请在这儿输入您的问题")

            if st.button("发送", key="predict"):
                with st.spinner("🤔 AI 正在思考，请稍等..."):
                    history = st.session_state.get('state')
                    st.session_state["state"] = st_chat(
                        text, history, container=container,
                        previous_messages=['请上传需要分析的PDF，我将为你解答'],
                        reply_func=reply_func,
                    )

            with st.expander('点击可查看被召回的知识'):
                st.dataframe(qa.recall.drop('embedding', 1, errors='ignore'))
                # st.dataframe(qa.recall)


    with tabs[1]:
        if bytes_array:
            display_pdf(base64_pdf)
        else:
            st.warning('### 请先上传PDF')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__
# @Time         : 2023/5/26 13:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


def load_llm4chat(model_name_or_path="THUDM/chatglm-6b", device='cpu', num_gpus=2, model_base=None, **kwargs):
    if not model_base:  # 模型基座
        model_base = Path(model_name_or_path).name.lower()
        for p in Path(__file__).parent.glob('*.py'):
            if p.stem in model_base:
                # logger.warning(p) # 自动推断模型基座
                model_base = p.stem

    model_base = importlib.import_module(f"chatllm.llms.{model_base}")
    return model_base.load_llm4chat(
        model_name_or_path=model_name_or_path,
        device=device,
        num_gpus=num_gpus,
        **kwargs)


if __name__ == '__main__':
    # print(load_llm4chat('/Users/betterme/PycharmProjects/AI/CHAT_MODEL/chatglm-6b'))
    print(Path(__file__).parent)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2021/1/31 10:20 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

cli = typer.Typer(name="ChatLLM CLI")

if LOCAL_HOST.startswith('10.219'):
    MODEL_PATH = "/CHAT_MODEL/chatglm-6b"


def f(a=1, **kw):
    print(a)
    print(kw)


@cli.command(help="help")  # help会覆盖docstring
def clitest(**kwargs):  # 不支持 **kwargs
    f(**kwargs)


@cli.command()  # help会覆盖docstring
def webui(name: str = 'chatpdf', port=8501):
    """
        chatllm-run webui --name chatpdf --port 8501
    """
    main = get_resolve_path(f'../webui/{name}.py', __file__)
    os.system(f'streamlit run {main} --server.port {port}')


@cli.command()  # help会覆盖docstring
def flask_api(model_name_or_path=None, host='127.0.0.1', port=8000, path='/'):
    """
        chatllm-run flask-api --model_name_or_path <MODEL_PATH> --host 127.0.0.1 --port 8000
    """
    from chatllm.applications import ChatBase

    qa = ChatBase()
    qa.load_llm4chat(model_name_or_path or MODEL_PATH)
    qa.run_serving(host, port, path)


@cli.command()  # help会覆盖docstring
def openapi(llm_model, host='127.0.0.1', port: int = 8000, debug='1'):
    """
        chatllm-run openapi <MODEL_PATH> --host 127.0.0.1 --port 8000
    """

    os.environ['LLM_MODEL'] = llm_model
    os.environ['DEBUG'] = debug

    from meutils.serving.fastapi import App
    from chatllm.api.routes.api import router

    app = App()
    app.include_router(router)
    app.run(host=host, port=port)


if __name__ == '__main__':
    cli()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatglm
# @Time         : 2023/5/19 17:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import torch
from transformers import AutoTokenizer, AutoModel

# ME
from meutils.pipe import *
from chatllm.utils.gpu_utils import load_chatglm_on_gpus


def load_llm(model_name_or_path="THUDM/chatglm-6b", device='cpu', num_gpus=2):
    model = AutoModel.from_pretrained(model_name_or_path, trust_remote_code=True)
    tokenizer = AutoTokenizer.from_pretrained(model_name_or_path, trust_remote_code=True)

    if torch.cuda.is_available() and device.lower().startswith("cuda"):
        num_gpus = min(num_gpus, torch.cuda.device_count())

        if num_gpus == 1:  # 单卡
            model = model.half().cuda()
            # model.transformer.prefix_encoder.float()
        elif 'chatglm' in model_name_or_path:  # chatglm多卡
            model = load_chatglm_on_gpus(model_name_or_path, num_gpus)

    else:
        model = model.float().to(device)

    return model.eval(), tokenizer


def load_llm4chat(model_name_or_path="THUDM/chatglm-6b", device='cpu', num_gpus=2):
    model, tokenizer = load_llm(model_name_or_path, device, num_gpus)

    def stream_chat(query, history=None, return_history=True, **chat_kwargs):
        """
        for i in chat('1+1', return_history=False):
            print(i, end='')
        """
        # chat_kwargs 标准化: max_tokens, temperature, top_p
        chat_kwargs['max_length'] = max(2048, chat_kwargs.get('max_tokens', 2048))

        idx = 0
        for response, history in model.stream_chat(tokenizer=tokenizer, query=query, history=history, **chat_kwargs):
            ret = response[idx:]
            if ret[-1:] == "\uFFFD":
                continue

            idx = len(response)
            if return_history:
                yield ret, history
            else:
                yield ret

    return stream_chat


if __name__ == '__main__':
    for i in load_llm4chat('/CHAT_MODEL/chatglm-6b')('你好', return_history=False):
        print(i, end='')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : Question2Answer
# @Time         : 2023/4/21 12:25
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.decorators import clear_cuda_cache

from chatllm.utils import DEVICE
from chatllm.llms import load_llm4chat


class ChatBase(object):

    def __init__(self, **kwargs):
        self.do_chat = None

        self.history = []
        self.knowledge_base = None
        self.role = None

        # 重写 chat函数会更好 prompt += "[Round {}]\n问：{}\n答：{}\n".format(i, old_query, response) # 根据角色配置模板
        self.prompt_template = os.environ.get('PROMPT_TEMPLATE', '{role}')

    def __call__(self, **kwargs):
        return self.qa(**kwargs)

    def qa(self, query, **kwargs):
        """可重写"""
        return self._qa(query, **kwargs)

    @clear_cuda_cache(bins=int(os.getenv('GPU_TIME_INTERVAL', 2)))
    def _qa(self, query, knowledge_base='', role='', max_turns=1, return_history=False):
        self.role = role or os.environ.get('LLM_ROLE', '')
        self.knowledge_base = str(knowledge_base).strip()
        if self.knowledge_base:
            self.query = self.prompt_template.format(context=self.knowledge_base, question=query, role='')
        else:
            self.query = """{role}\n请回答以下问题\n{question}""".format(question=query, role=self.role)  # 知识库为空则转通用回答

        global history
        _history = history[-(max_turns - 1):] if max_turns > 1 else []  # 截取最大轮次
        for _ in self.do_chat(query=self.query.strip(), history=_history, return_history=return_history):
            yield _  # (response, history)

    def load_llm(self, model_name_or_path="THUDM/chatglm-6b", device=DEVICE, return_history=False, **kwargs):
        self.do_chat = load_llm4chat(model_name_or_path, device=device, **kwargs)
        self.do_chat = partial(self.do_chat, return_history=return_history)

    def set_chat_kwargs(self, **kwargs):
        self.do_chat = partial(self.do_chat, **kwargs)


if __name__ == '__main__':
    from chatllm.applications import ChatBase

    qa = ChatBase()
    qa.load_llm(model_name_or_path="/CHAT_MODEL/chatglm-6b")

    for _ in qa(query='你是谁', return_history=False):
        print(_, end='')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : base
# @Time         : 2023/5/26 10:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from fastapi import APIRouter, Body, Depends, HTTPException

router = APIRouter()


@router.get("/")
def read_root():
    return {"Hello World!": "ChatLLM"}


@router.get("/v1/models")
def get_models():
    ret = {"data": [], "object": "list"}
    ret['data'].append({
        "created": 1677610602,
        "id": "gpt-3.5-turbo",
        "object": "model",
        "owned_by": "openai",
        "permission": [
            {
                "created": 1680818747,
                "id": "modelperm-fTUZTbzFp7uLLTeMSo9ks6oT",
                "object": "model_permission",
                "allow_create_engine": False,
                "allow_sampling": True,
                "allow_logprobs": True,
                "allow_search_indices": False,
                "allow_view": True,
                "allow_fine_tuning": False,
                "organization": "*",
                "group": None,
                "is_blocking": False
            }
        ],
        "root": "gpt-3.5-turbo",
        "parent": None,
    })
    ret['data'].append({
        "created": 1671217299,
        "id": "text-embedding-ada-002",
        "object": "model",
        "owned_by": "openai-internal",
        "permission": [
            {
                "created": 1678892857,
                "id": "modelperm-Dbv2FOgMdlDjO8py8vEjD5Mi",
                "object": "model_permission",
                "allow_create_engine": False,
                "allow_sampling": True,
                "allow_logprobs": True,
                "allow_search_indices": True,
                "allow_view": True,
                "allow_fine_tuning": False,
                "organization": "*",
                "group": None,
                "is_blocking": False
            }
        ],
        "root": "text-embedding-ada-002",
        "parent": ""
    })

    return ret



#!/usr/bin/env python

"""The setup script."""
import time
import pandas as pd
from pathlib import Path
from setuptools import setup, find_packages

DIR = Path(__file__).resolve().parent
version = time.strftime("%Y.%m.%d.%H.%M.%S", time.localtime())

with open('README.md') as readme_file:
    readme = readme_file.read()


get_requirements = lambda p='requirements.txt': pd.read_csv(p, comment='#', names=['name']).name.tolist()
extras_require = {v.name.split('_')[1][:-4]: get_requirements(v) for v in DIR.glob('requirements_*')}
extras_require['all'] = list(set(sum(extras_require.values(), [])))

setup(
    author="yuanjie",
    author_email='313303303@qq.com',
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Create a Python package.",
    entry_points={
        'console_scripts': [
            'chatllm-run=chatllm.clis.cli:cli'
        ],
    },
    setup_requires=["pandas"],
    install_requires=get_requirements(),
    extras_require=extras_require,  # pip install -U meutils\[all\]
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='chatllm',
    # name='ChatLLM',
    name='OpenBMB', # 抢占包

    packages=find_packages(include=['chatllm', 'chatllm.*']),

    test_suite='tests',
    url='https://github.com/yuanjie-ai/ChatLLM',
    version=version,  # '0.0.0',
    zip_safe=False,
)
